'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var compositionApi = require('@vue/composition-api');
var FormulateForm = _interopDefault(require('@braid/vue-formulate/src/FormulateForm'));

class EventBus {
  constructor() {
    this.store = new Map();
  }

  on(type, handler) {
    const handlers = this.store.get(type);
    const added = handlers && handlers.push(handler);
    if (!added) {
      this.store.set(type, [handler]);
    }
  }

  off(type, handler) {
    const handlers = this.store.get(type);
    const index = handlers.indexOf(handler);
    if (handlers && index > -1) {
      handlers.splice(index, 1);
    }
  }

  emit(type, payload) {
    this.store
      .get(type)
      .slice()
      .map((handler) => handler(payload));
  }
}

/**
 * Given an object and an index, complete an object for schema-generation.
 * @param {object} item
 * @param {int} index
 */
function leaf(item, index, { nodeHook, eventBus, componentHook } = {}) {
  if (item && typeof item === 'object' && !Array.isArray(item)) {
    const { children = null, component = 'FormulateInput', depth = 1, events = [], on = {}, ...attrs } = item;
    const type = component === 'FormulateInput' ? attrs.type || 'text' : '';
    const name = attrs.name || type || 'el';
    const key = attrs.id || `${name}-${depth}-${index}`;
    const els = Array.isArray(children) ? children.map((child) => Object.assign(child, { depth: depth + 1 })) : children;
    const onExtended = {
      ...on,
      ...events.reduce((onEvents, eventName) => {
        onEvents[eventName] = function (element) {
          if (on[eventName]) on[eventName](payload);
          eventBus.emit('events', { name: eventName, node: { name, type, key }, element });
        };
        return onEvents
      }, {}),
    };
    return nodeHook(Object.assign({ type, key, depth, component, definition: { attrs, on: onExtended }, children: tree(els, { nodeHook, eventBus, componentHook }) }))
  }
  return null
}

/**
 * Recursive function to create vNodes from a schema.
 * @param {Functon} h createElement
 * @param {Array|string} schema
 */
function tree(schema, { componentHook, nodeHook, eventBus } = {}) {
  if (Array.isArray(schema)) {
    return schema.map((el, idx) => {
      const item = leaf(el, idx, { nodeHook, eventBus, componentHook });
      return componentHook(item)
    })
  }
  return schema
}

const FormulateSchema = {
  functional: false,
  props: {
    schema: FormulateForm.props.schema,
    nodeHook: {
      type: Function,
      default: (node) => node,
    },
    componentHook: {
      type: Function,
      default: (node) => compositionApi.createElement(node.component, node.definition, node.children),
    },
  },
  setup(props, { emit }) {
    const eventBus = new EventBus();
    eventBus.on('events', (payload) => emit('events', payload));

    return () => compositionApi.createElement('div', {}, tree(props.schema, { nodeHook: props.nodeHook, componentHook: props.componentHook }))
  },
};

//

var script = {
  ...FormulateForm,
  props: {
    ...FormulateForm.props,
    nodeHook: FormulateSchema.props.nodeHook,
    componentHook: FormulateSchema.props.componentHook
  }
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "form",
    {
      class: _vm.classes,
      on: {
        submit: function($event) {
          $event.preventDefault();
          return _vm.formSubmitted($event)
        }
      }
    },
    [
      _vm.schema
        ? _c("FormulateSchema", {
            attrs: {
              schema: _vm.schema,
              nodeHook: _vm.nodeHook,
              componentHook: _vm.componentHook
            },
            on: {
              events: function(payload) {
                return _vm.$emit("events", payload)
              }
            }
          })
        : _vm._e(),
      _vm._v(" "),
      !_vm.hasFormErrorObservers
        ? _c("FormulateErrors", { attrs: { context: _vm.formContext } })
        : _vm._e(),
      _vm._v(" "),
      _vm._t("default")
    ],
    2
  )
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );

function plugin(instance) {
  instance.extend({
    components: {
      FormulateForm: __vue_component__,
      FormulateSchema,
    },
  });
}

exports.plugin = plugin;
